@@ -114,7 +114,7 @@ flags:
 			break flags
 		}
 	}
-	format, width, hasWidth := parseFieldWidth(format)
+	format, width, hasWidth := parseFieldWidth(format, args)
 	if hasWidth {
 		spec += strconv.Itoa(width)
 	}
@@ -506,7 +506,7 @@ more:
 // nonexistent or small field width cause truncation of a field; if the result
 // of a conversion is wider than the field width, the field is expanded to
 // contain the conversion result.
-func parseFieldWidth(format uintptr) (_ uintptr, n int, ok bool) {
+func parseFieldWidth(format uintptr, args *uintptr) (_ uintptr, n int, ok bool) {
 	first := true
 	for {
 		var digit int
@@ -514,7 +514,13 @@ func parseFieldWidth(format uintptr) (_ uintptr, n int, ok bool) {
 		case first && c == '0':
 			return format, n, ok
 		case first && c == '*':
-			panic(todo(""))
+			format++
+			switch c := *(*byte)(unsafe.Pointer(format)); {
+			case c >= '0' && c <= '9':
+				panic(todo(""))
+			default:
+				return format, int(VaInt32(args)), true
+			}
 		case c >= '0' && c <= '9':
 			format++
 			ok = true
